<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

require_once "WxPay.Exception.php";
require_once "WxPay.Config.php";
require_once "WxPay.Data.php";

/**
 * 
 * 接口访问类，包含所有微信支付API列表的封装，类中方法为static方法，
 * 每个接口有默认超时时间（除提交被扫支付为10s，上报超时时间为1s外，其他均为6s）
 *
 */
class WechatPaymentApi
{
	/**
	 * 
	 * 统一下单，WechatPaymentUnifiedOrder中out_trade_no、body、total_fee、trade_type必填
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WechatPaymentUnifiedOrder $inputObj
	 * @param int $timeOut
	 * @throws WechatPaymentException
	 * @return 成功时返回，其他抛异常
	 */
	public static function unifiedOrder($inputObj, $timeOut = 60,$WxCfg)
	{
		$url = "https://api.mch.weixin.qq.com/pay/unifiedorder";
		//检测必填参数
		if(!$inputObj->IsOut_trade_noSet()) {
			throw new WechatPaymentException("缺少统一支付接口必填参数out_trade_no！");
		}else if(!$inputObj->IsBodySet()){
			throw new WechatPaymentException("缺少统一支付接口必填参数body！");
		}else if(!$inputObj->IsTotal_feeSet()) {
			throw new WechatPaymentException("缺少统一支付接口必填参数total_fee！");
		}else if(!$inputObj->IsTrade_typeSet()) {
			throw new WechatPaymentException("缺少统一支付接口必填参数trade_type！");
		}
		
		//关联参数
		if($inputObj->GetTrade_type() == "JSAPI" && !$inputObj->IsOpenidSet()){
			throw new WechatPaymentException("统一支付接口中，缺少必填参数openid！trade_type为JSAPI时，openid为必填参数！");
		}
		if($inputObj->GetTrade_type() == "NATIVE" && !$inputObj->IsProduct_idSet()){
			throw new WechatPaymentException("统一支付接口中，缺少必填参数product_id！trade_type为JSAPI时，product_id为必填参数！");
		}
		
		//异步通知url未设置，则使用配置文件中的url
		if(!$inputObj->IsNotify_urlSet()){
			$inputObj->SetNotify_url('');//异步通知url
		}
		
		$inputObj->SetAppid($WxCfg->getAPPID());//公众账号ID
		$inputObj->SetMch_id($WxCfg->getMCHID());//商户号
		//$inputObj->SetSpbill_create_ip($_SERVER['REMOTE_ADDR']);//终端ip
		//$inputObj->SetSpbill_create_ip("1.1.1.1");  	    
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串
		//签名
        $inputObj->SetSign($WxCfg);
        $xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();//请求开始时间
		$response = self::postXmlCurl($xml, $url, false, $timeOut,$WxCfg);
		$result = WechatPaymentResults::Init($response,$WxCfg);
		self::reportCostTime($url, $startTimeStamp, $result,$WxCfg);//上报请求花费时间
		
		return $result;
	}

	
	/**
	 * 
	 * 测速上报，该方法内部封装在report中，使用时请注意异常流程
	 * WechatPaymentReport中interface_url、return_code、result_code、user_ip、execute_time_必填
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WechatPaymentReport $inputObj
	 * @param int $timeOut
	 * @throws WechatPaymentException
	 * @return 成功时返回，其他抛异常
	 */
	public static function report($inputObj, $timeOut = 60,$WxCfg)
	{
		$url = "https://api.mch.weixin.qq.com/payitil/report";
		//检测必填参数
		if(!$inputObj->IsInterface_urlSet()) {
			throw new WechatPaymentException("接口URL，缺少必填参数interface_url！");
		} if(!$inputObj->IsReturn_codeSet()) {
			throw new WechatPaymentException("返回状态码，缺少必填参数return_code！");
		} if(!$inputObj->IsResult_codeSet()) {
			throw new WechatPaymentException("业务结果，缺少必填参数result_code！");
		} if(!$inputObj->IsUser_ipSet()) {
			throw new WechatPaymentException("访问接口IP，缺少必填参数user_ip！");
		} if(!$inputObj->IsExecute_time_Set()) {
			throw new WechatPaymentException("接口耗时，缺少必填参数execute_time_！");
		}
		$inputObj->SetAppid($WxCfg->getAPPID());//公众账号ID
		$inputObj->SetMch_id($WxCfg->getMCHID());//商户号
		$inputObj->SetUser_ip($_SERVER['REMOTE_ADDR']);//终端ip
		$inputObj->SetTime(date("YmdHis"));//商户上报时间	 
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串
		
		$inputObj->SetSign($WxCfg);//签名
		$xml = $inputObj->ToXml();
		
		$startTimeStamp = self::getMillisecond();//请求开始时间
		$response = self::postXmlCurl($xml, $url, false, $timeOut,$WxCfg);
		return $response;
	}
	
	/**
	 * 
	 * 生成二维码规则,模式一生成支付二维码
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WechatPaymentBizPayUrl $inputObj
	 * @param int $timeOut
	 * @throws WechatPaymentException
	 * @return 成功时返回，其他抛异常
	 */
	public static function bizpayurl($inputObj, $timeOut = 60,$WxCfg)
	{
		if(!$inputObj->IsProduct_idSet()){
			throw new WechatPaymentException("生成二维码，缺少必填参数product_id！");
		}
		
		$inputObj->SetAppid($WxCfg->getAPPID());//公众账号ID
		$inputObj->SetMch_id($WxCfg->getMCHID());//商户号
		$inputObj->SetTime_stamp(time());//时间戳	 
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串
		
		$inputObj->SetSign($WxCfg);//签名
		
		return $inputObj->GetValues();
	}

    /**
     *
     * 查询订单，WechatPaymentOrderQuery中out_trade_no、transaction_id至少填一个
     * appid、mchid、spbill_create_ip、nonce_str不需要填入
     * @param WechatPaymentOrderQuery $inputObj
     * @param int $timeOut
     * @throws WechatPaymentException
     * @return 成功时返回，其他抛异常
     */
    public static function orderQuery($inputObj, $WxCfg,$timeOut = 60)
    {
        $url = "https://api.mch.weixin.qq.com/pay/orderquery";
        //检测必填参数
        if(!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {
            throw new WechatPaymentException("订单查询接口中，out_trade_no、transaction_id至少填一个！");
        }
        $inputObj->SetAppid($WxCfg->getAPPID());//公众账号ID
        $inputObj->SetMch_id($WxCfg->getMCHID());//商户号
        $inputObj->SetNonce_str(self::getNonceStr());//随机字符串

        $inputObj->SetSign($WxCfg);//签名
        $xml = $inputObj->ToXml();

        $startTimeStamp = self::getMillisecond();//请求开始时间
        $response = self::postXmlCurl($xml, $url, false, $timeOut,$WxCfg);
        $result = WechatPaymentResults::Init($response,$WxCfg);
        self::reportCostTime($url, $startTimeStamp, $result,$WxCfg);//上报请求花费时间

        return $result;
    }

	
	/**
	 * 
	 * 产生随机字符串，不长于32位
	 * @param int $length
	 * @return 产生的随机字符串
	 */
	public static function getNonceStr($length = 32) 
	{
		$chars = "abcdefghijklmnopqrstuvwxyz0123456789";  
		$str ="";
		for ( $i = 0; $i < $length; $i++ )  {  
			$str .= substr($chars, mt_rand(0, strlen($chars)-1), 1);  
		} 
		return $str;
	}
	
	/**
	 * 直接输出xml
	 * @param string $xml
	 */
	public static function replyNotify($xml)
	{
		echo $xml;
	}
	
	/**
	 * 
	 * 上报数据， 上报的时候将屏蔽所有异常流程
	 * @param string $usrl
	 * @param int $startTimeStamp
	 * @param array $data
	 */
	private static function reportCostTime($url, $startTimeStamp, $data,$WxCfg)
	{
		//如果不需要上报数据
		if($WxCfg->getREPORTLEVENL() == 0){
			return;
		} 
		//如果仅失败上报
		if($WxCfg->getREPORTLEVENL() == 1 &&
			 array_key_exists("return_code", $data) &&
			 $data["return_code"] == "SUCCESS" &&
			 array_key_exists("result_code", $data) &&
			 $data["result_code"] == "SUCCESS")
		 {
		 	return;
		 }
		 
		//上报逻辑
		$endTimeStamp = self::getMillisecond();
		$objInput = new WechatPaymentReport();
		$objInput->SetInterface_url($url);
		$objInput->SetExecute_time_($endTimeStamp - $startTimeStamp);
		//返回状态码
		if(array_key_exists("return_code", $data)){
			$objInput->SetReturn_code($data["return_code"]);
		}
		//返回信息
		if(array_key_exists("return_msg", $data)){
			$objInput->SetReturn_msg($data["return_msg"]);
		}
		//业务结果
		if(array_key_exists("result_code", $data)){
			$objInput->SetResult_code($data["result_code"]);
		}
		//错误代码
		if(array_key_exists("err_code", $data)){
			$objInput->SetErr_code($data["err_code"]);
		}
		//错误代码描述
		if(array_key_exists("err_code_des", $data)){
			$objInput->SetErr_code_des($data["err_code_des"]);
		}
		//商户订单号
		if(array_key_exists("out_trade_no", $data)){
			$objInput->SetOut_trade_no($data["out_trade_no"]);
		}
		//设备号
		if(array_key_exists("device_info", $data)){
			$objInput->SetDevice_info($data["device_info"]);
		}
		
		try{
			self::report($objInput,1,$WxCfg);
		} catch (WechatPaymentException $e){
			//不做任何处理
		}
	}

	/**
	 * 以post方式提交xml到对应的接口url
	 * 
	 * @param string $xml  需要post的xml数据
	 * @param string $url  url
	 * @param bool $useCert 是否需要证书，默认不需要
	 * @param int $second   url执行超时时间，默认30s
	 * @param WechatPaymentConfig $WxCfg
	 * @throws WechatPaymentException
	 */
	private static function postXmlCurl($xml, $url, $useCert = false, $second = 60,$WxCfg)
	{		
		$ch = curl_init();
		//设置超时
		curl_setopt($ch, CURLOPT_TIMEOUT, $second);
		
		//如果有配置代理这里就设置代理
		if($WxCfg->isEnableProxy()){
			curl_setopt($ch,CURLOPT_PROXY, $WxCfg->getCURLPROXYHOST());
			curl_setopt($ch,CURLOPT_PROXYPORT, $WxCfg->getCURLPROXYPORT());
		}
		curl_setopt($ch,CURLOPT_URL, $url);
		curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);
        //curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,TRUE);
	//	curl_setopt($ch,CURLOPT_SSL_VERIFYHOST,2);//严格校验
		//设置header
		curl_setopt($ch, CURLOPT_HEADER, FALSE);
		//要求结果为字符串且输出到屏幕上
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);

		if($useCert == true){
			//设置证书
			//使用证书：cert 与 key 分别属于两个.pem文件
			curl_setopt($ch,CURLOPT_SSLCERTTYPE,'PEM');
			curl_setopt($ch,CURLOPT_SSLCERT, $WxCfg->getSSLCERTPATH());
			curl_setopt($ch,CURLOPT_SSLKEYTYPE,'PEM');
			curl_setopt($ch,CURLOPT_SSLKEY,$WxCfg->getSSLKEYPATH());
		}
		//post提交方式
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		//运行curl
		$data = curl_exec($ch);
		//返回结果
		if($data){
			curl_close($ch);
			return $data;
		} else {
			$error = curl_error($ch);
			curl_close($ch);
			throw new WechatPaymentException("curl出错，错误码:$error");
		}
	}
	
	
	/**
	 *
	 * 申请退款，WechatPaymentRefund中out_trade_no、transaction_id至少填一个且
	 * out_refund_no、total_fee、refund_fee、op_user_id为必填参数
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WechatPaymentRefund $inputObj
	 * @param int $timeOut
	 * @param WechatPaymentConfig $WxCfg
	 * @throws XH_Wx_Pay_Exception
	 * @return 成功时返回，其他抛异常
	 */
	public static function refund($inputObj, $timeOut = 60,$WxCfg)
	{
		$url = "https://api.mch.weixin.qq.com/secapi/pay/refund";
		//检测必填参数
		if(!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {
			throw new XH_Wx_Pay_Exception("退款申请接口中，out_trade_no、transaction_id至少填一个！");
		}else if(!$inputObj->IsOut_refund_noSet()){
			throw new XH_Wx_Pay_Exception("退款申请接口中，缺少必填参数out_refund_no！");
		}else if(!$inputObj->IsTotal_feeSet()){
			throw new XH_Wx_Pay_Exception("退款申请接口中，缺少必填参数total_fee！");
		}else if(!$inputObj->IsRefund_feeSet()){
			throw new XH_Wx_Pay_Exception("退款申请接口中，缺少必填参数refund_fee！");
		}else if(!$inputObj->IsOp_user_idSet()){
			throw new XH_Wx_Pay_Exception("退款申请接口中，缺少必填参数op_user_id！");
		}
		$inputObj->SetAppid($WxCfg->getAPPID());//公众账号ID
		$inputObj->SetMch_id($WxCfg->getMCHID());//商户号
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串
	
		$inputObj->SetSign($WxCfg);//签名
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();//请求开始时间
		$response = self::postXmlCurl($xml, $url, true, $timeOut,$WxCfg);
		$result = WechatPaymentResults::Init($response,$WxCfg);
		self::reportCostTime($url, $startTimeStamp, $result,$WxCfg);//上报请求花费时间
	
		return $result;
	}
	
	
	/**
	 * 获取毫秒级别的时间戳
	 */
	private static function getMillisecond()
	{
		//获取毫秒的时间戳
		$time = explode ( " ", microtime () );
		$time = $time[1] . ($time[0] * 1000);
		$time2 = explode( ".", $time );
		$time = $time2[0];
		return $time;
	}
}

